@@ -33,7 +33,7 @@ from nemo.collections.nlp.models.machine_translation.mt_enc_dec_config import MT
 from nemo.collections.nlp.models.machine_translation.mt_enc_dec_model import MTEncDecModel
 from nemo.collections.nlp.modules.common.transformer import AttentionBridge, TopKSequenceGenerator
 from nemo.core.classes.common import typecheck
-from nemo.utils import logging, model_utils
+from nemo.utils import logging, model_utils, timers
 
 __all__ = ['MTBottleneckModel']
 
@@ -75,6 +75,7 @@ class MTBottleneckModel(MTEncDecModel):
         self.latent_size: int = cfg.get("latent_size", -1)
         self.non_recon_warmup_batches: int = cfg.get("non_recon_warmup_batches", 200000)
         self.recon_per_token: bool = cfg.get("recon_per_token", True)
+        self.log_timing: bool = cfg.get("log_timing", True)
 
         # if True, translation uses the mean of latent for VAE and MIM
         self.deterministic_translate = True
@@ -250,7 +251,7 @@ class MTBottleneckModel(MTEncDecModel):
             loss = -(log_p_x_given_z + warmup_coef * loss_terms)
 
             info_dict["log_q_z_given_x"] = log_q_z_given_x.detach().cpu()
-            info_dict["log_var_q_z_given_x"] = z_logv.detach().cpu()
+            info_dict["log_var_q_z_given_x"] = z_logv.detach().mean().cpu()
             info_dict["log_p_z"] = log_p_z.detach().cpu()
             info_dict["kl_div_q_p"] = (log_q_z_given_x - log_p_z).detach().cpu()
 
@@ -263,7 +264,7 @@ class MTBottleneckModel(MTEncDecModel):
             return loss
 
     @typecheck()
-    def forward(self, src, src_mask, tgt, tgt_mask):
+    def forward(self, src, src_mask, tgt, tgt_mask, timer=None):
         """
         return_info - if True, returns loss, info_dict with additional information
                       regarding the loss that can be logged
@@ -273,12 +274,20 @@ class MTBottleneckModel(MTEncDecModel):
             self.test_encoder_ids(src, raise_error=True)
             self.test_decoder_ids(tgt, raise_error=True)
 
+        if timer is not None:
+            timer.start("encoder")
         enc_hiddens, enc_mask = self.encoder(input_ids=src, encoder_mask=src_mask, return_mask=True,)
 
         # build posterior distribution q(x|z)
         z, z_mean, z_logv = self.encode_latent(hidden=enc_hiddens)
         z_mask = enc_mask
 
+        if timer is not None:
+            timer.stop("encoder")
+
+        if timer is not None:
+            timer.start("decoder")
+
         # decoding cross attention context
         context_hiddens = self.latent2hidden(z)
 
@@ -289,10 +298,15 @@ class MTBottleneckModel(MTEncDecModel):
         # build decoding distribution
         tgt_log_probs = self.log_softmax(hidden_states=tgt_hiddens)
 
+        if timer is not None:
+            timer.stop("decoder")
+
         return z, z_mean, z_logv, z_mask, tgt_log_probs
 
     @torch.no_grad()
-    def batch_translate(self, src: torch.LongTensor, src_mask: torch.LongTensor, return_beam_scores: bool = False):
+    def batch_translate(
+        self, src: torch.LongTensor, src_mask: torch.LongTensor, return_beam_scores: bool = False, cache={}
+    ):
         """
         Translates a minibatch of inputs from source language to target language.
         Args:
@@ -303,16 +317,28 @@ class MTBottleneckModel(MTEncDecModel):
             inputs: a list of string containing detokenized inputs
         """
         mode = self.training
+        timer = cache.get("timer", None)
         try:
             self.eval()
-            enc_hiddens, enc_mask = self.encoder(input_ids=src, encoder_mask=src_mask, return_mask=True)
 
             # build posterior distribution q(x|z)
-            z, z_mean, _ = self.encode_latent(hidden=enc_hiddens)
+            if ("z" not in cache) or ("z_mean" not in cache) or ("z_mask" not in cache):
+                if timer is not None:
+                    timer.start("encoder")
+                enc_hiddens, enc_mask = self.encoder(input_ids=src, encoder_mask=src_mask, return_mask=True)
+                z, z_mean, _ = self.encode_latent(hidden=enc_hiddens)
+                if timer is not None:
+                    timer.stop("encoder")
+            else:
+                enc_mask = cache["z_mask"]
+                z = cache["z"]
+                z_mean = cache["z_mean"]
 
             if getattr(self, "deterministic_translate", True):
                 z = z_mean
 
+            if timer is not None:
+                timer.start("sampler")
             # decoding cross attention context
             context_hiddens = self.latent2hidden(z)
 
@@ -321,6 +347,9 @@ class MTBottleneckModel(MTEncDecModel):
                 encoder_input_mask=enc_mask,
                 return_beam_scores=return_beam_scores,
             )
+            if timer is not None:
+                timer.stop("sampler")
+
             if return_beam_scores:
                 all_translations, scores, best_translations = best_translations
                 scores = scores.view(-1)
@@ -376,6 +405,11 @@ class MTBottleneckModel(MTEncDecModel):
         return {'loss': train_loss, 'log': tensorboard_logs}
 
     def eval_step(self, batch, batch_idx, mode, dataloader_idx=0):
+        if self.log_timing:
+            timer = timers.NamedTimer()
+        else:
+            timer = None
+
         for i in range(len(batch)):
             if batch[i].ndim == 3:
                 # Dataset returns already batched data and the first dimension of size 1 added by DataLoader
@@ -387,7 +421,7 @@ class MTBottleneckModel(MTEncDecModel):
             self.target_processor = self.target_processor_list[dataloader_idx]
 
         src_ids, src_mask, tgt_ids, tgt_mask, labels = batch
-        z, z_mean, z_logv, z_mask, tgt_log_probs = self(src_ids, src_mask, tgt_ids, tgt_mask)
+        z, z_mean, z_logv, z_mask, tgt_log_probs = self(src_ids, src_mask, tgt_ids, tgt_mask, timer=timer)
         eval_loss, info_dict = self.loss(
             z=z,
             z_mean=z_mean,
@@ -400,8 +434,10 @@ class MTBottleneckModel(MTEncDecModel):
             train=False,
             return_info=True,
         )
-        # this will run encoder twice -- TODO: potentially fix
-        _, translations = self.batch_translate(src=src_ids, src_mask=src_mask)
+        # pass cache to sampler in order to reuse encoder's output
+        cache = dict(z=z, z_mean=z_mean, z_mask=z_mask, timer=timer,)
+
+        _, translations = self.batch_translate(src=src_ids, src_mask=src_mask, cache=cache)
 
         num_measurements = labels.shape[0] * labels.shape[1]
         if dataloader_idx == 0:
@@ -417,9 +453,16 @@ class MTBottleneckModel(MTEncDecModel):
         ground_truths = [self.target_processor.detokenize(tgt.split(' ')) for tgt in ground_truths]
         num_non_pad_tokens = np.not_equal(np_tgt, self.decoder_tokenizer.pad_id).sum().item()
 
+        # collect logs
+        log_dict = {k: v.detach().cpu().numpy() if torch.is_tensor(v) else v for k, v in info_dict.items()}
+        # add timing if required
+        if timer is not None:
+            for k, v in timer.export().items():
+                log_dict[f"{k}_timing"] = v
+
         return {
             'translations': translations,
             'ground_truths': ground_truths,
             'num_non_pad_tokens': num_non_pad_tokens,
-            'log': {k: v.detach().cpu().numpy() if torch.is_tensor(v) else v for k, v in info_dict.items()},
+            'log': log_dict,
         }
