@@ -46,6 +46,14 @@ MEGATRON_CACHE = os.path.join(torch_home, "megatron")
 CONFIGS = {"345m": {"hidden_size": 1024, "num_attention_heads": 16, "num_layers": 24, "max_position_embeddings": 512}}
 
 MEGATRON_CONFIG_MAP = {
+    "megatron-gpt-345m": {
+        "config": CONFIGS["345m"],
+        "checkpoint": "models/nvidia/megatron_lm_345m/versions/v0.0/files/release/mp_rank_00/model_optim_rng.pt",
+        "vocab": "https://s3.amazonaws.com/models.huggingface.co/bert/gpt2-vocab.json",
+        "merges_file": "https://s3.amazonaws.com/models.huggingface.co/bert/gpt2-merges.txt",
+        "do_lower_case": False,
+        "tokenizer_name": "gpt2",
+    },
     "megatron-bert-345m-uncased": {
         "config": CONFIGS["345m"],
         "checkpoint": "https://api.ngc.nvidia.com/v2/models/nvidia/megatron_bert_345m/versions/v0.0/files/release/mp_rank_00/model_optim_rng.pt",
@@ -97,6 +105,7 @@ def get_megatron_lm_model(
     config_file: Optional[str] = None,
     checkpoint_file: Optional[str] = None,
     vocab_file: Optional[str] = None,
+    merges_file: Optional[str] = None,
 ) -> Tuple[MegatronBertEncoder, str]:
     """
     Returns MegatronBertEncoder and a default or user specified path to the checkpoint file
@@ -113,81 +122,87 @@ def get_megatron_lm_model(
         model: MegatronBertEncoder
         checkpoint_file: path to checkpoint file or directory
     """
-    config = None
-    # get default config and checkpoint
-    if config_file:
-        with open(config_file) as f:
-            config = json.load(f)
-            # replace dashes with underscores in config keys
-            fixed_config = {}
-            for key in config.keys():
-                fixed_key = key.replace("-", "_")
-                if fixed_key == 'max_seq_length':
-                    fixed_key = 'max_position_embeddings'
-                fixed_config[fixed_key] = config[key]
-            # 'vocab_size" no longer used.
-            if 'vocab_size' in fixed_config:
-                fixed_config.pop('vocab_size')
-            config = fixed_config
-    elif config_dict:
-        config = config_dict
-    elif pretrained_model_name in get_megatron_lm_models_list():
-        config = get_megatron_config(pretrained_model_name)
-    else:
-        raise ValueError(f"{pretrained_model_name} is not supported")
-
-    if config is None:
-        raise ValueError(f"config_file or config_dict is required for {pretrained_model_name}")
-
-    if not checkpoint_file:
-        checkpoint_file = get_megatron_checkpoint(pretrained_model_name)
-
-    if not vocab_file:
-        vocab_file = get_megatron_vocab_file(pretrained_model_name)
-
-    app_state = AppState()
-    if app_state.model_parallel_size is not None and app_state.model_parallel_rank is not None:
-        # model parallel already known from .nemo restore
-        model_parallel_size = app_state.model_parallel_size
-        model_parallel_rank = app_state.model_parallel_rank
-    elif os.path.isdir(checkpoint_file):
-        # starting training from megatron-lm checkpoint
-        mp_ranks = glob.glob(os.path.join(checkpoint_file, 'mp_rank*'))
-        model_parallel_size = len(mp_ranks)
-        app_state.model_parallel_size = model_parallel_size
-        logging.info(
-            (
-                f'restore_path: {checkpoint_file} is a directory. '
-                f'Assuming megatron model parallelism with '
-                f'model_parallel_size: {model_parallel_size}'
-            )
-        )
-        # try to get local rank from global
-        local_rank = None
-        try:
-            local_rank = int(os.environ['LOCAL_RANK'])
-        except:
-            logging.info('Global variable LOCAL_RANK not yet specified')
-        if local_rank is not None:
-            app_state.local_rank = local_rank
-        else:
-            # if local is None then we are on the main process
-            local_rank = 0
-        model_parallel_rank = compute_model_parallel_rank(local_rank, model_parallel_size)
-        app_state.model_parallel_rank = model_parallel_rank
-    else:
-        model_parallel_size = None
-        model_parallel_rank = None
-
-    model = MegatronBertEncoder(
-        model_name=pretrained_model_name,
-        config=config,
-        vocab_file=vocab_file,
-        model_parallel_size=model_parallel_size,
-        model_parallel_rank=model_parallel_rank,
+    raise ValueError(
+        f'megatron-lm bert has been deprecated in NeMo 1.5. Please use an earlier release of NeMo. Megatron bert support will be added back to NeMo in a future release.'
     )
+    # config = None
+    # # get default config and checkpoint
+    # if config_file:
+    #     with open(config_file) as f:
+    #         config = json.load(f)
+    #         # replace dashes with underscores in config keys
+    #         fixed_config = {}
+    #         for key in config.keys():
+    #             fixed_key = key.replace("-", "_")
+    #             if fixed_key == 'max_seq_length':
+    #                 fixed_key = 'max_position_embeddings'
+    #             fixed_config[fixed_key] = config[key]
+    #         # 'vocab_size" no longer used.
+    #         if 'vocab_size' in fixed_config:
+    #             fixed_config.pop('vocab_size')
+    #         config = fixed_config
+    # elif config_dict:
+    #     config = config_dict
+    # elif pretrained_model_name in get_megatron_lm_models_list():
+    #     config = get_megatron_config(pretrained_model_name)
+    # else:
+    #     raise ValueError(f"{pretrained_model_name} is not supported")
 
-    return model, checkpoint_file
+    # if config is None:
+    #     raise ValueError(f"config_file or config_dict is required for {pretrained_model_name}")
+
+    # if not checkpoint_file:
+    #     checkpoint_file = get_megatron_checkpoint(pretrained_model_name)
+
+    # if not vocab_file:
+    #     vocab_file = get_megatron_vocab_file(pretrained_model_name)
+
+    # if not merges_file:
+    #     merges_file = get_megatron_merges_file(pretrained_model_name)
+
+    # app_state = AppState()
+    # if app_state.model_parallel_size is not None and app_state.model_parallel_rank is not None:
+    #     # model parallel already known from .nemo restore
+    #     model_parallel_size = app_state.model_parallel_size
+    #     model_parallel_rank = app_state.model_parallel_rank
+    # elif os.path.isdir(checkpoint_file):
+    #     # starting training from megatron-lm checkpoint
+    #     mp_ranks = glob.glob(os.path.join(checkpoint_file, 'mp_rank*'))
+    #     model_parallel_size = len(mp_ranks)
+    #     app_state.model_parallel_size = model_parallel_size
+    #     logging.info(
+    #         (
+    #             f'restore_path: {checkpoint_file} is a directory. '
+    #             f'Assuming megatron model parallelism with '
+    #             f'model_parallel_size: {model_parallel_size}'
+    #         )
+    #     )
+    #     # try to get local rank from global
+    #     local_rank = None
+    #     try:
+    #         local_rank = int(os.environ['LOCAL_RANK'])
+    #     except:
+    #         logging.info('Global variable LOCAL_RANK not yet specified')
+    #     if local_rank is not None:
+    #         app_state.local_rank = local_rank
+    #     else:
+    #         # if local is None then we are on the main process
+    #         local_rank = 0
+    #     model_parallel_rank = compute_model_parallel_rank(local_rank, model_parallel_size)
+    #     app_state.model_parallel_rank = model_parallel_rank
+    # else:
+    #     model_parallel_size = None
+    #     model_parallel_rank = None
+
+    # model = MegatronBertEncoder(
+    #     model_name=pretrained_model_name,
+    #     config=config,
+    #     vocab_file=vocab_file,
+    #     model_parallel_size=model_parallel_size,
+    #     model_parallel_rank=model_parallel_rank,
+    # )
+
+    # return model, checkpoint_file
 
 
 def compute_model_parallel_rank(local_rank, model_parallel_size):
@@ -239,6 +254,26 @@ def get_megatron_vocab_file(pretrained_model_name: str) -> str:
     return path
 
 
+def get_megatron_merges_file(pretrained_model_name: str) -> str:
+    """
+    Gets merge file from cache or downloads it
+
+    Args:
+        pretrained_model_name: pretrained model name
+
+    Returns:
+        path: path to the vocab file
+    """
+    if 'gpt' not in pretrained_model_name.lower():
+        return None
+    _check_megatron_name(pretrained_model_name)
+    url = MEGATRON_CONFIG_MAP[pretrained_model_name]["merges_file"]
+
+    path = os.path.join(MEGATRON_CACHE, pretrained_model_name + "_merges")
+    path = _download(path, url)
+    return path
+
+
 def get_megatron_checkpoint(pretrained_model_name: str) -> str:
     """
     Gets checkpoint file from cache or downloads it
