@@ -25,12 +25,28 @@ from nemo.collections.common.tokenizers.word_tokenizer import WordTokenizer
 from nemo.collections.common.tokenizers.youtokentome_tokenizer import YouTokenToMeTokenizer
 from nemo.collections.nlp.modules.common.huggingface.huggingface_utils import get_huggingface_pretrained_lm_models_list
 from nemo.collections.nlp.modules.common.lm_utils import get_pretrained_lm_models_list
-from nemo.collections.nlp.modules.common.megatron.megatron_utils import get_megatron_tokenizer
+from nemo.collections.nlp.parts.nlp_overrides import HAVE_APEX
 from nemo.utils import logging
 
+try:
+    from nemo.collections.nlp.modules.common.megatron.megatron_utils import get_megatron_tokenizer
+
+    HAVE_APEX = True
+
+except (ImportError, ModuleNotFoundError):
+    HAVE_APEX = False
+
+
 __all__ = ['get_tokenizer', 'get_tokenizer_list']
 
 
+megatron_tokenizer_model_map = {
+    'BertWordPieceLowerCase': 'megatron-bert-345m-uncased',
+    'BertWordPieceCase': 'megatron-bert-345m-cased',
+    'GPT2BPETokenizer': 'megatron-gpt-345m',
+}
+
+
 def get_tokenizer_list() -> List[str]:
     """
     Returns all all supported tokenizer names
@@ -57,6 +73,7 @@ def get_tokenizer(
     tokenizer_name: str,
     tokenizer_model: Optional[str] = None,
     vocab_file: Optional[str] = None,
+    merges_file: Optional[str] = None,
     special_tokens: Optional[Dict[str, str]] = None,
     use_fast: Optional[bool] = False,
     bpe_dropout: Optional[float] = 0.0,
@@ -80,10 +97,15 @@ def get_tokenizer(
         special_tokens_dict = special_tokens
 
     if 'megatron' in tokenizer_name:
+        if not HAVE_APEX:
+            raise RuntimeError("Apex required to use megatron.")
         if vocab_file is None:
             vocab_file = nemo.collections.nlp.modules.common.megatron.megatron_utils.get_megatron_vocab_file(
                 tokenizer_name
             )
+            merges_file = nemo.collections.nlp.modules.common.megatron.megatron_utils.get_megatron_merges_file(
+                tokenizer_name
+            )
         tokenizer_name = get_megatron_tokenizer(tokenizer_name)
 
     if tokenizer_name == 'sentencepiece':
@@ -101,7 +123,11 @@ def get_tokenizer(
         f"Getting HuggingFace AutoTokenizer with pretrained_model_name: {tokenizer_name}, vocab_file: {vocab_file}, special_tokens_dict: {special_tokens_dict}, and use_fast: {use_fast}"
     )
     return AutoTokenizer(
-        pretrained_model_name=tokenizer_name, vocab_file=vocab_file, **special_tokens_dict, use_fast=use_fast
+        pretrained_model_name=tokenizer_name,
+        vocab_file=vocab_file,
+        merges_file=merges_file,
+        **special_tokens_dict,
+        use_fast=use_fast,
     )
 
 
@@ -110,6 +136,7 @@ def get_nmt_tokenizer(
     model_name: Optional[str] = None,
     tokenizer_model: Optional[str] = None,
     vocab_file: Optional[str] = None,
+    merges_file: Optional[str] = None,
     special_tokens: Optional[Dict[str, str]] = None,
     use_fast: Optional[bool] = False,
     bpe_dropout: Optional[float] = 0.0,
@@ -138,10 +165,14 @@ def get_nmt_tokenizer(
     elif library == 'huggingface':
         logging.info(f'Getting HuggingFace AutoTokenizer with pretrained_model_name: {model_name}')
         return AutoTokenizer(
-            pretrained_model_name=model_name, vocab_file=vocab_file, **special_tokens_dict, use_fast=use_fast
+            pretrained_model_name=model_name,
+            vocab_file=vocab_file,
+            merges_file=merges_file,
+            **special_tokens_dict,
+            use_fast=use_fast,
         )
     elif library == 'sentencepiece':
-        logging.info(f'Getting SentencePiece with model: {model_name}')
+        logging.info(f'Getting SentencePiece with model: {tokenizer_model}')
         return nemo.collections.common.tokenizers.sentencepiece_tokenizer.SentencePieceTokenizer(
             model_path=tokenizer_model, special_tokens=special_tokens_dict
         )
@@ -149,10 +180,12 @@ def get_nmt_tokenizer(
         logging.info(f'Using byte-level tokenization')
         return ByteLevelTokenizer()
     elif library == 'megatron':
+        if model_name in megatron_tokenizer_model_map:
+            model_name = megatron_tokenizer_model_map[model_name]
         logging.info(
             f'Getting Megatron tokenizer for pretrained model name: {model_name} and custom vocab file: {vocab_file}'
         )
-        return get_tokenizer(tokenizer_name=model_name, vocab_file=vocab_file)
+        return get_tokenizer(tokenizer_name=model_name, vocab_file=vocab_file, merges_file=merges_file)
     else:
         raise NotImplementedError(
             'Currently we only support "yttm", "huggingface", "sentencepiece", "megatron", and "byte-level" tokenizer libraries.'
