@@ -38,6 +38,7 @@ from nemo.core.neural_types.elements import (
     RegressionValuesType,
     TokenDurationType,
     TokenIndex,
+    TokenLogDurationType,
 )
 from nemo.core.neural_types.neural_type import NeuralType
 from nemo.utils import logging
@@ -81,8 +82,14 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
         self.log_train_images = False
         self.mel_loss = MelLoss()
         loss_scale = 0.1 if self.learn_alignment else 1.0
-        self.pitch_loss = PitchLoss(loss_scale=loss_scale)
-        self.duration_loss = DurationLoss(loss_scale=loss_scale)
+        dur_loss_scale = loss_scale
+        pitch_loss_scale = loss_scale
+        if "dur_loss_scale" in cfg:
+            dur_loss_scale = cfg.dur_loss_scale
+        if "pitch_loss_scale" in cfg:
+            pitch_loss_scale = cfg.pitch_loss_scale
+        self.pitch_loss = PitchLoss(loss_scale=pitch_loss_scale)
+        self.duration_loss = DurationLoss(loss_scale=dur_loss_scale)
         input_fft_kwargs = {}
         if self.learn_alignment:
             self.aligner = instantiate(self._cfg.alignment_module)
@@ -110,6 +117,7 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
             cfg.pitch_embedding_kernel_size,
             cfg.n_mel_channels,
         )
+        self._input_types = self._output_types = None
 
     @property
     def tb_logger(self):
@@ -131,8 +139,9 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
             return self._parser
 
         if self.learn_alignment:
-            vocab = AudioToCharWithDursF0Dataset.make_vocab(**self._cfg.train_ds.dataset.vocab)
-            self._parser = vocab.encode
+            if self.vocab is None:
+                self.vocab = AudioToCharWithDursF0Dataset.make_vocab(**self._cfg.train_ds.dataset.vocab)
+            self._parser = self.vocab.encode
         else:
             self._parser = parsers.make_parser(
                 labels=self._cfg.labels,
@@ -372,7 +381,7 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
         list_of_models = []
         model = PretrainedModelInfo(
             pretrained_model_name="tts_en_fastpitch",
-            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/tts_en_fastpitch/versions/1.0.0/files/tts_en_fastpitch.nemo",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/tts_en_fastpitch/versions/1.4.0/files/tts_en_fastpitch_align.nemo",
             description="This model is trained on LJSpeech sampled at 22050Hz with and can be used to generate female English voices with an American accent.",
             class_=cls,
         )
@@ -380,35 +389,65 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
 
         return list_of_models
 
-    @property
-    def input_module(self):
-        return self.fastpitch
+    ### Export code
+    def input_example(self):
+        """
+        Generates input examples for tracing etc.
+        Returns:
+            A tuple of input examples.
+        """
+        par = next(self.fastpitch.parameters())
+        inp = torch.randint(
+            0, self.fastpitch.encoder.word_emb.num_embeddings, (1, 44), device=par.device, dtype=torch.int64
+        )
+        pitch = torch.randn((1, 44), device=par.device, dtype=torch.float32) * 0.5
+        pace = torch.clamp((torch.randn((1, 44), device=par.device, dtype=torch.float32) + 1) * 0.1, min=0.01)
+
+        inputs = {'text': inp, 'pitch': pitch, 'pace': pace}
+
+        if self.fastpitch.speaker_emb is not None:
+            inputs['speaker'] = torch.randint(
+                0, self.fastpitch.speaker_emb.num_embeddings, (1,), device=par.device, dtype=torch.int64
+            )
+
+        return (inputs,)
+
+    def forward_for_export(self, text, pitch, pace, speaker=None):
+        return self.fastpitch.infer(text=text, pitch=pitch, pace=pace, speaker=speaker)
 
     @property
-    def output_module(self):
-        return self.fastpitch
+    def input_types(self):
+        return self._input_types
 
-    def forward_for_export(self, text):
-        (
-            spect,
-            num_frames,
-            durs_predicted,
-            log_durs_predicted,
-            pitch_predicted,
-            attn_soft,
-            attn_logprob,
-            attn_hard,
-            attn_hard_dur,
-            pitch,
-        ) = self.fastpitch(text=text)
-        return spect.to(torch.float), num_frames, durs_predicted, log_durs_predicted, pitch_predicted
+    @property
+    def output_types(self):
+        return self._output_types
+
+    def _prepare_for_export(self, **kwargs):
+        super()._prepare_for_export(**kwargs)
+
+        # Define input_types and output_types as required by export()
+        self._input_types = {
+            "text": NeuralType(('B', 'T'), TokenIndex()),
+            "pitch": NeuralType(('B', 'T'), RegressionValuesType()),
+            "pace": NeuralType(('B', 'T'), optional=True),
+            "speaker": NeuralType(('B'), Index()),
+        }
+        self._output_types = {
+            "spect": NeuralType(('B', 'D', 'T'), MelSpectrogramType()),
+            "num_frames": NeuralType(('B'), TokenDurationType()),
+            "durs_predicted": NeuralType(('B', 'T'), TokenDurationType()),
+            "log_durs_predicted": NeuralType(('B', 'T'), TokenLogDurationType()),
+            "pitch_predicted": NeuralType(('B', 'T'), RegressionValuesType()),
+        }
+
+    def _export_teardown(self):
+        self._input_types = self._output_types = None
 
     @property
     def disabled_deployment_input_names(self):
         """Implement this method to return a set of input names disabled for export"""
-        return set(["durs", "pitch", "speaker", "pace", "spec", "attn_prior", "mel_lens", "input_lens"])
-
-    @property
-    def disabled_deployment_output_names(self):
-        """Implement this method to return a set of input names disabled for export"""
-        return set(["attn_soft", "pitch", "attn_logprob", "attn_hard", "attn_hard_dur",])
+        disabled_inputs = set()
+        if self.fastpitch.speaker_emb is None:
+            disabled_inputs.add("speaker")
+        return disabled_inputs
