@@ -243,7 +243,8 @@ class FastPitchModule(NeuralModule):
         # Predict pitch
         pitch_predicted = self.pitch_predictor(enc_out, enc_mask)
         if pitch is not None:
-            if self.learn_alignment:
+            if self.learn_alignment and pitch.shape[-1] != pitch_predicted.shape[-1]:
+                # Pitch during training is per spectrogram frame, but during inference, it should be per character
                 pitch = average_pitch(pitch.unsqueeze(1), attn_hard_dur).squeeze(1)
             pitch_emb = self.pitch_emb(pitch.unsqueeze(1))
         else:
@@ -275,13 +276,27 @@ class FastPitchModule(NeuralModule):
             pitch,
         )
 
-    def input_example(self):
-        """
-        Generates input examples for tracing etc.
-        Returns:
-            A tuple of input examples.
-        """
-        par = next(self.parameters())
-        inp = torch.randint(0, self.encoder.word_emb.num_embeddings, (1, 44), device=par.device, dtype=torch.int64)
+    def infer(self, *, text, pitch=None, speaker=None, pace=1.0):
+        # Calculate speaker embedding
+        if self.speaker_emb is None or speaker is None:
+            spk_emb = 0
+        else:
+            spk_emb = self.speaker_emb(speaker).unsqueeze(1)
 
-        return ({'text': inp},)
+        # Input FFT
+        enc_out, enc_mask = self.encoder(input=text, conditioning=spk_emb)
+
+        # Predict duration and pitch
+        log_durs_predicted = self.duration_predictor(enc_out, enc_mask)
+        durs_predicted = torch.clamp(torch.exp(log_durs_predicted) - 1, 0, self.max_token_duration)
+        pitch_predicted = self.pitch_predictor(enc_out, enc_mask) + pitch
+        pitch_emb = self.pitch_emb(pitch_predicted.unsqueeze(1))
+        enc_out = enc_out + pitch_emb.transpose(1, 2)
+
+        # Expand to decoder time dimension
+        len_regulated, dec_lens = regulate_len(durs_predicted, enc_out, pace)
+
+        # Output FFT
+        dec_out, _ = self.decoder(input=len_regulated, seq_lens=dec_lens)
+        spect = self.proj(dec_out).transpose(1, 2)
+        return spect.to(torch.float), dec_lens, durs_predicted, log_durs_predicted, pitch_predicted
