@@ -166,7 +166,7 @@ class Exportable(ABC):
             input_names = self._process_input_names()
             output_names = self._process_output_names()
 
-            output_example = self.forward(*input_list, **input_dict)
+            output_example = tuple(self.forward(*input_list, **input_dict))
 
             with torch.jit.optimized_execution(True), torch.no_grad():
                 jitted_model = self._try_jit_compile_model(self, try_script)
@@ -195,7 +195,14 @@ class Exportable(ABC):
 
                     # Verify the model can be read, and is valid
                     self._verify_onnx_export(
-                        output, output_example, input_list, input_dict, input_names, check_tolerance, check_trace
+                        output,
+                        output_example,
+                        input_list,
+                        input_dict,
+                        input_names,
+                        output_names,
+                        check_tolerance,
+                        check_trace,
                     )
                 else:
                     raise ValueError(f'Encountered unknown export format {format}.')
@@ -203,10 +210,11 @@ class Exportable(ABC):
             typecheck.set_typecheck_enabled(enabled=True)
             if forward_method:
                 type(self).forward = old_forward_method
+            self._export_teardown()
         return ([output], [output_descr])
 
     def _verify_onnx_export(
-        self, output, output_example, input_list, input_dict, input_names, check_tolerance, check_trace
+        self, output, output_example, input_list, input_dict, input_names, output_names, check_tolerance, check_trace
     ):
         onnx_model = onnx.load(output)
         onnx.checker.check_model(onnx_model, full_check=True)
@@ -218,11 +226,11 @@ class Exportable(ABC):
 
         if test_runtime:
             self._verify_runtime(
-                onnx_model, input_list, input_dict, input_names, output_example, output, check_tolerance
+                onnx_model, input_list, input_dict, input_names, output_names, output_example, output, check_tolerance
             )
 
     def _verify_runtime(
-        self, onnx_model, input_list, input_dict, input_names, output_example, output, check_tolerance
+        self, onnx_model, input_list, input_dict, input_names, output_names, output_example, output, check_tolerance
     ):
         try:
             import onnxruntime
@@ -231,14 +239,17 @@ class Exportable(ABC):
             return
 
         sess = onnxruntime.InferenceSession(onnx_model.SerializeToString())
-        ort_out = sess.run(None, to_onnxrt_input(input_names, input_list, input_dict))
+        ort_out = sess.run(output_names, to_onnxrt_input(input_names, input_list, input_dict))
         all_good = True
 
-        for out_name, out in enumerate(ort_out):
-            expected = output_example[out_name].cpu()
-            if not torch.allclose(torch.from_numpy(out), expected, rtol=check_tolerance, atol=100 * check_tolerance):
-                all_good = False
-                logging.info(f"onnxruntime results mismatch! PyTorch(expected):\n{expected}\nONNXruntime:\n{out}")
+        for i, out in enumerate(ort_out[0]):
+            expected = output_example[i]
+            if torch.is_tensor(expected):
+                if not torch.allclose(
+                    torch.from_numpy(out), expected.cpu(), rtol=check_tolerance, atol=100 * check_tolerance
+                ):
+                    all_good = False
+                    logging.info(f"onnxruntime results mismatch! PyTorch(expected):\n{expected}\nONNXruntime:\n{out}")
         status = "SUCCESS" if all_good else "FAIL"
         logging.info(f"ONNX generated at {output} verified with onnxruntime : " + status)
 
@@ -365,6 +376,12 @@ class Exportable(ABC):
         replace_1D_2D = kwargs.get('replace_1D_2D', False)
         replace_for_export(self, replace_1D_2D)
 
+    def _export_teardown(self):
+        """
+        Override this method for any teardown code after export.
+        """
+        pass
+
     def _wrap_forward_method(self):
         old_forward_method = None
 
