@@ -160,6 +160,87 @@ class WarmupHoldPolicy(WarmupPolicy):
         return self._get_lr(step)
 
 
+class WarmupAnnealHoldPolicy(_LRScheduler):
+    """Adds warmup kwargs and warmup logic to lr policy.
+    All arguments should be passed as kwargs for clarity,
+    Args:
+        warmup_steps: Number of training steps in warmup stage
+        warmup_ratio: Ratio of warmup steps to total steps
+        max_steps: Total number of steps while training or `None` for
+            infinite training
+        min_lr: Minimum lr to hold the learning rate after decay at.
+        constant_steps: Number of steps to keep lr constant at.
+        constant_ratio: Ratio of steps to keep lr constant.
+    """
+
+    def __init__(
+        self,
+        optimizer,
+        *,
+        warmup_steps=None,
+        warmup_ratio=None,
+        constant_steps=None,
+        constant_ratio=None,
+        max_steps=None,
+        min_lr=0.0,
+        last_epoch=-1,
+    ):
+        assert not (
+            warmup_steps is not None and warmup_ratio is not None
+        ), "Either use particular number of step or ratio"
+        assert not (
+            constant_steps is not None and constant_ratio is not None
+        ), "Either use constant_steps or constant_ratio"
+        assert warmup_ratio is None or max_steps is not None, "If there is a ratio, there should be a total steps"
+
+        # It is necessary to assign all attributes *before* __init__,
+        # as class is wrapped by an inner class.
+        self.max_steps = max_steps
+
+        if warmup_steps is not None:
+            self.warmup_steps = warmup_steps
+        elif warmup_ratio is not None:
+            self.warmup_steps = int(warmup_ratio * max_steps)
+        else:
+            self.warmup_steps = 0
+
+        if constant_steps is not None:
+            self.constant_steps = constant_steps
+            self.decay_steps = max_steps - (self.constant_steps + self.warmup_steps)
+            assert self.decay_steps > 0
+        elif constant_ratio is not None:
+            self.constant_steps = int(constant_ratio * max_steps)
+            self.decay_steps = max_steps - (self.constant_steps + self.warmup_steps)
+            assert self.decay_steps > 0
+        else:
+            self.constant_steps = 0
+            self.decay_steps = max_steps - self.warmup_steps
+
+        self.min_lr = min_lr
+        super().__init__(optimizer, last_epoch)
+
+    def get_lr(self):
+        if not self._get_lr_called_within_step:
+            warnings.warn(
+                "To get the last learning rate computed by the scheduler, please use `get_last_lr()`.", UserWarning
+            )
+
+        step = self.last_epoch
+
+        if step <= self.warmup_steps:
+            lr_val = (step + 1) / (self.warmup_steps + 1)
+            return [initial_lr * lr_val for initial_lr in self.base_lrs]
+
+        if step > self.max_steps:
+            return [self.min_lr for _ in self.base_lrs]
+
+        return self._get_lr(step)
+
+    def _get_lr(self, step):
+        """Simple const lr policy"""
+        return self.base_lrs
+
+
 def _squareroot_annealing(initial_lr, step, max_steps, min_lr):
     mult = ((max_steps - step) / max_steps) ** 0.5
     out_lr = initial_lr * mult
@@ -180,6 +261,30 @@ def _cosine_annealing(initial_lr, step, max_steps, min_lr):
     return out_lr
 
 
+def _linear_warmup_with_cosine_annealing(max_lr, warmup_steps, step, decay_steps, min_lr):
+
+    assert max_lr > min_lr
+    # Use linear warmup for the initial part.
+    if warmup_steps > 0 and step <= warmup_steps:
+        return max_lr * float(step) / float(warmup_steps)
+
+    # For any steps larger than `decay_steps`, use `min_lr`.
+    if step > warmup_steps + decay_steps:
+        return min_lr
+
+    # If we are done with the warmup period, use the decay style.
+    num_steps_ = step - warmup_steps
+    decay_steps_ = decay_steps
+    decay_ratio = float(num_steps_) / float(decay_steps_)
+    assert decay_ratio >= 0.0
+    assert decay_ratio <= 1.0
+    delta_lr = max_lr - min_lr
+
+    coeff = 0.5 * (math.cos(math.pi * decay_ratio) + 1.0)
+
+    return min_lr + coeff * delta_lr
+
+
 def _poly_decay(initial_lr, step, decay_steps, power, min_lr, cycle):
     if cycle:
         multiplier = 1.0 if step == 0 else math.ceil(step / decay_steps)
@@ -221,7 +326,7 @@ class SquareRootAnnealing(WarmupPolicy):
         return new_lrs
 
 
-class CosineAnnealing(WarmupPolicy):
+class CosineAnnealing(WarmupAnnealHoldPolicy):
     def __init__(self, optimizer, *, max_steps, min_lr=0, last_epoch=-1, **kwargs):
         super().__init__(optimizer=optimizer, max_steps=max_steps, last_epoch=last_epoch, min_lr=min_lr, **kwargs)
 
@@ -232,15 +337,27 @@ class CosineAnnealing(WarmupPolicy):
                     f"{self} received an initial learning rate that was lower than the minimum learning rate."
                 )
 
-        new_lrs = [
-            _cosine_annealing(
-                initial_lr=initial_lr,
-                step=step - self.warmup_steps,
-                max_steps=self.max_steps - self.warmup_steps,
-                min_lr=self.min_lr,
-            )
-            for initial_lr in self.base_lrs
-        ]
+        if self.constant_steps is None:
+            new_lrs = [
+                _cosine_annealing(
+                    initial_lr=initial_lr,
+                    step=step - self.warmup_steps,
+                    max_steps=self.max_steps - self.warmup_steps,
+                    min_lr=self.min_lr,
+                )
+                for initial_lr in self.base_lrs
+            ]
+        else:
+            new_lrs = [
+                _linear_warmup_with_cosine_annealing(
+                    max_lr=self.base_lrs[0],
+                    warmup_steps=self.warmup_steps,
+                    step=step,
+                    decay_steps=self.decay_steps,
+                    min_lr=self.min_lr,
+                )
+                for _ in self.base_lrs
+            ]
         return new_lrs
 
 
@@ -587,7 +704,17 @@ def prepare_lr_scheduler(
 
         # Compute effective num max_steps
         num_samples = len(train_dataloader.dataset)
-        batch_size = train_dataloader.batch_size
+        # TODO: not sure if this will be the correct LR schedule for Megatron
+        # we may need to override ModelPT setup_optimization
+        if train_dataloader.batch_size is not None:
+            batch_size = train_dataloader.batch_size
+        elif hasattr(train_dataloader, 'batch_sampler') and train_dataloader.batch_sampler is not None:
+            if train_dataloader.batch_sampler.micro_batch_size is not None:
+                batch_size = train_dataloader.batch_sampler.micro_batch_size
+            else:
+                raise ValueError(f'Could not find batch_size from batch_sampler: {train_dataloader.batch_sampler}')
+        else:
+            raise ValueError(f'Could not find batch_size from train_dataloader: {train_dataloader}')
         drop_last = train_dataloader.drop_last
 
         max_steps = compute_max_steps(
@@ -662,8 +789,6 @@ def compute_max_steps(
     elif steps_per_epoch != float('inf'):
         # limit_train_batches is a percentage of batches per epoch
         steps_per_epoch = int(steps_per_epoch * limit_train_batches)
-        if accumulate_grad_batches == 1:
-            steps_per_epoch = max(steps_per_epoch, 1)
 
     return math.ceil(steps_per_epoch / accumulate_grad_batches) * max_epochs
 
