@@ -19,6 +19,7 @@ import sys
 import time
 from copy import deepcopy
 from dataclasses import dataclass
+from datetime import timedelta
 from pathlib import Path
 from shutil import copy, move
 from typing import Any, Dict, List, Optional, Union
@@ -28,14 +29,16 @@ from hydra.core.hydra_config import HydraConfig
 from hydra.utils import get_original_cwd
 from omegaconf import DictConfig, OmegaConf, open_dict
 from pytorch_lightning.callbacks import Callback, ModelCheckpoint
+from pytorch_lightning.callbacks.timer import Interval, Timer
 from pytorch_lightning.loggers import LoggerCollection as _LoggerCollection
 from pytorch_lightning.loggers import TensorBoardLogger, WandbLogger
 from pytorch_lightning.plugins.training_type.ddp import DDPPlugin
-from pytorch_lightning.utilities import rank_zero_only
+from pytorch_lightning.trainer.states import RunningStage
+from pytorch_lightning.utilities.distributed import rank_zero_info
 from pytorch_lightning.utilities.types import _METRIC
 
 from nemo.constants import NEMO_ENV_VARNAME_VERSION
-from nemo.utils import app_state, logging, timers
+from nemo.utils import logging, timers
 from nemo.utils.app_state import AppState
 from nemo.utils.exceptions import NeMoBaseException
 from nemo.utils.get_rank import is_global_rank_zero
@@ -72,12 +75,12 @@ class CallbackParams:
     save_top_k: Optional[int] = 3
     save_weights_only: Optional[bool] = False
     mode: Optional[str] = "min"
-    period: Optional[int] = None
     every_n_val_epochs: Optional[int] = 1
     prefix: Optional[str] = None  # If None, exp_manager will attempt to handle the filepath
     postfix: str = ".nemo"
     save_best_model: bool = False
     always_save_nemo: bool = False
+    model_parallel_size: Optional[int] = None
 
 
 @dataclass
@@ -85,6 +88,8 @@ class StepTimingParams:
     reduction: Optional[str] = "mean"
     # if True torch.cuda.synchronize() is called on start/stop
     sync_cuda: Optional[bool] = False
+    # if positive, defines the size of a sliding window for computing mean
+    buffer_size: Optional[int] = -1
 
 
 @dataclass
@@ -111,6 +116,7 @@ class ExpManagerConfig:
     # logs timing of train/val/test steps
     log_step_timing: Optional[bool] = True
     step_timing_kwargs: Optional[StepTimingParams] = StepTimingParams()
+    model_parallel_size: Optional[int] = None
 
 
 class TimingCallback(Callback):
@@ -122,7 +128,10 @@ class TimingCallback(Callback):
         self.timer = timers.NamedTimer(**timer_kwargs)
 
     def _on_batch_start(self, name):
-        self.timer.reset(name)
+        # reset only if we do not return mean of a sliding window
+        if self.timer.buffer_size <= 0:
+            self.timer.reset(name)
+
         self.timer.start(name)
 
     def _on_batch_end(self, name, pl_module):
@@ -184,7 +193,7 @@ def exp_manager(trainer: 'pytorch_lightning.Trainer', cfg: Optional[Union[DictCo
                 lightning's TensorboardLogger system of using version_{int}.
             - use_datetime_version (bool): Whether to use a datetime string for version. Defaults to True.
             - resume_if_exists (bool): Whether this experiment is resuming from a previous run. If True, it sets
-                trainer.checkpoint_connector.resume_checkpoint_path so that the trainer should auto-resume. exp_manager will move files
+                trainer.checkpoint_connector.resume_from_checkpoint_fit_path so that the trainer should auto-resume. exp_manager will move files
                 under log_dir to log_dir/run_{int}. Defaults to False. From v1.0.0, when resume_if_exists is True,
                 we would not create version folders to make it easier to find the log folder for next runs.
             - resume_past_end (bool): exp_manager errors out if resume_if_exists is True and a checkpoint matching
@@ -363,7 +372,7 @@ def check_resume(
     resume_ignore_no_checkpoint: bool = False,
 ):
     """Checks that resume=True was used correctly with the arguments pass to exp_manager. Sets
-    trainer.checkpoint_connector.resume_checkpoint_path as necessary.
+    trainer.checkpoint_connector.resume_from_checkpoint_fit_path as necessary.
 
     Returns:
         log_dir (Path): the log_dir
@@ -375,6 +384,7 @@ def check_resume(
         NotFoundError: If resume is True, resume_ignore_no_checkpoint is False, and checkpoints could not be found.
         ValueError: If resume is True, and there were more than 1 checkpoint could found.
     """
+
     if not log_dir:
         raise ValueError(f"Resuming requires the log_dir {log_dir} to be passed to exp_manager")
 
@@ -418,7 +428,7 @@ def check_resume(
         logging.info(f"Resuming from {last_checkpoints[0]}")
         checkpoint = last_checkpoints[0]
 
-    trainer.checkpoint_connector.resume_checkpoint_path = str(checkpoint)
+    trainer.checkpoint_connector.resume_from_checkpoint_fit_path = str(checkpoint)
 
     if is_global_rank_zero():
         # Check to see if any files exist that need to be moved
@@ -644,12 +654,21 @@ class NeMoModelCheckpoint(ModelCheckpoint):
     """ Light wrapper around Lightning's ModelCheckpoint to force a saved checkpoint on train_end
     """
 
-    def __init__(self, always_save_nemo=False, save_best_model=False, postfix=".nemo", n_resume=False, **kwargs):
+    def __init__(
+        self,
+        always_save_nemo=False,
+        save_best_model=False,
+        postfix=".nemo",
+        n_resume=False,
+        model_parallel_size=None,
+        **kwargs,
+    ):
         # Parse and store "extended" parameters: save_best model and postfix.
         self.always_save_nemo = always_save_nemo
         self.save_best_model = save_best_model
         self.postfix = postfix
         self.previous_best_path = ""
+        self.model_parallel_size = model_parallel_size
 
         # `prefix` is deprecated
         if 'prefix' in kwargs:
@@ -661,6 +680,7 @@ class NeMoModelCheckpoint(ModelCheckpoint):
         super().__init__(**kwargs)
 
         if self.save_top_k != -1 and n_resume:
+            logging.debug("Checking previous runs")
             self.nemo_topk_check_previous_run()
 
     def nemo_topk_check_previous_run(self):
@@ -669,7 +689,6 @@ class NeMoModelCheckpoint(ModelCheckpoint):
             self.kth_best_model_path
             self.best_model_score
             self.best_model_path
-            self._del_model
         except AttributeError:
             raise AttributeError("Lightning's ModelCheckpoint was updated. NeMoModelCheckpoint will need an update.")
         self.best_k_models = {}
@@ -679,6 +698,8 @@ class NeMoModelCheckpoint(ModelCheckpoint):
 
         checkpoints = list(Path(self.dirpath).rglob("*.ckpt"))
         for checkpoint in checkpoints:
+            if self.model_parallel_size is not None and self.model_parallel_size > 1:
+                checkpoint = self._uninject_mp_rank(checkpoint)
             checkpoint = str(checkpoint)
             if checkpoint[-10:] == '-last.ckpt':
                 continue
@@ -697,130 +718,96 @@ class NeMoModelCheckpoint(ModelCheckpoint):
 
         ### This section should be ok as rank zero will delete all excess checkpoints, since all other ranks are
         ### instantiated after rank zero. models_to_delete should be 0 for all other ranks.
-        models_to_delete = len(best_k_models) - self.save_top_k
+        if self.model_parallel_size is not None:
+            models_to_delete = len(best_k_models) - self.model_parallel_size * self.save_top_k
+        else:
+            models_to_delete = len(best_k_models) - self.save_top_k
         logging.debug(f'Number of models to delete: {models_to_delete}')
         for _ in range(models_to_delete):
             model = best_k_models.pop(-1)
             self.best_k_models.pop(model)
-            self._del_model(model)
+            self._del_model_without_trainer(model)
             logging.debug(f"Removed checkpoint: {model}")
 
         self.kth_best_model_path = best_k_models[-1]
         self.best_model_path = best_k_models[0]
         self.best_model_score = self.best_k_models[self.best_model_path]
 
-    @rank_zero_only
+    @staticmethod
+    def _uninject_mp_rank(filepath):
+        dirname = os.path.dirname(os.path.dirname(filepath))
+        basename = os.path.basename(filepath)
+        filepath = os.path.join(dirname, basename)
+        return filepath
+
     def on_save_checkpoint(self, trainer, pl_module, checkpoint):
         output = super().on_save_checkpoint(trainer, pl_module, checkpoint)
-
         if not self.always_save_nemo:
             return output
 
-        # Load the best model and then re-save it
-        app_state = AppState()
-        # since we are creating tarfile artifacts we need to update .nemo path
-        app_state.model_restore_path = os.path.abspath(
-            os.path.expanduser(os.path.join(self.dirpath, self.prefix + self.postfix))
-        )
-        if self.save_best_model:
-            if not os.path.exists(self.best_model_path):
-                return output
-
-            if self.best_model_path == self.previous_best_path:
-                return output
-
-            self.previous_model_path = self.best_model_path
-            old_state_dict = deepcopy(pl_module.state_dict())
-            checkpoint = torch.load(self.best_model_path, map_location='cpu')
-            if 'state_dict' in checkpoint:
-                checkpoint = checkpoint['state_dict']
-            # get a new instanace of the model
-            pl_module.load_state_dict(checkpoint, strict=True)
-            pl_module.save_to(save_path=app_state.model_restore_path)
-            pl_module.load_state_dict(old_state_dict, strict=True)
         else:
-            pl_module.save_to(save_path=app_state.model_restore_path)
-        return output
+            # Load the best model and then re-save it
+            app_state = AppState()
+            if app_state.model_parallel_size is not None and app_state.model_parallel_size > 1:
+                raise ValueError(f'always_save_nemo is not implemented for model parallel models.')
+            # since we are creating tarfile artifacts we need to update .nemo path
+            app_state.model_restore_path = os.path.abspath(
+                os.path.expanduser(os.path.join(self.dirpath, self.prefix + self.postfix))
+            )
+            if self.save_best_model:
+                if not os.path.exists(self.best_model_path):
+                    return output
+
+                if self.best_model_path == self.previous_best_path:
+                    return output
+
+                self.previous_model_path = self.best_model_path
+                old_state_dict = deepcopy(pl_module.state_dict())
+                checkpoint = torch.load(self.best_model_path, map_location='cpu')
+                if 'state_dict' in checkpoint:
+                    checkpoint = checkpoint['state_dict']
+                # get a new instanace of the model
+                pl_module.load_state_dict(checkpoint, strict=True)
+                pl_module.save_to(save_path=app_state.model_restore_path)
+                pl_module.load_state_dict(old_state_dict, strict=True)
+            else:
+                pl_module.save_to(save_path=app_state.model_restore_path)
+            return output
 
-    @rank_zero_only
     def on_train_end(self, trainer, pl_module):
         if trainer.fast_dev_run:
             return None
-        app_state = AppState()
-        if app_state.model_parallel_size is not None:
-            return None
 
-        # TODO: make this work for model parallel, need to call on data parallel rank 0 and update best_model_path
+        # Call parent on_train_end() to save the -last checkpoint
+        super().on_train_end(trainer, pl_module)
+
         # Load the best model and then re-save it
         if self.save_best_model:
-            trainer.checkpoint_connector.restore(self.best_model_path)
+            if self.best_model_path == "":
+                logging.warning(
+                    f"{self} was told to save the best checkpoint at the end of training, but no saved checkpoints "
+                    "were found. Saving latest model instead."
+                )
+            else:
+                trainer.checkpoint_connector.restore(self.best_model_path)
+
         pl_module.save_to(save_path=os.path.join(self.dirpath, self.prefix + self.postfix))
 
-    def _del_model(self, trainer: "pl.Trainer", filepath: str) -> None:
-        """ Overrides PTL method to account for model parallel checkpoints.
-            Updates checkpoint path based on model parallel rank.
-        """
+    def _del_model_without_trainer(self, filepath: str) -> None:
         app_state = AppState()
-        if app_state.model_parallel_size is not None:
+        if app_state.model_parallel_size is not None and app_state.model_parallel_size > 1:
             # filepath needs to be updated to include mp_rank
             dirname = os.path.dirname(filepath)
             basename = os.path.basename(filepath)
             filepath = f'{dirname}/mp_rank_{app_state.model_parallel_rank:02d}/{basename}'
 
-            # each model parallel rank needs to remove its model
-            if app_state.data_parallel_rank == 0:
-                super()._del_model(trainer, filepath)
-                logging.info(f"Removed model parallel checkpoint: {filepath}")
-
-        else:
-            return super()._del_model(trainer, filepath)
-
-    def _save_last_checkpoint(self, trainer: 'pl.Trainer', monitor_candidates: Dict[str, _METRIC]) -> None:
-        """ Overrides PTL method to account for model parallel checkpoints.
-            Checks for data parallel rank 0 rather than global rank 0.
-        """
-        app_state = AppState()
-        if app_state.model_parallel_size is not None:
-            if not self.save_last:
-                return
-
-            filepath = self._format_checkpoint_name(self.CHECKPOINT_NAME_LAST, monitor_candidates)
-            filepath = os.path.join(self.dirpath, f"{filepath}{self.FILE_EXTENSION}")
-
-            self._save_model(trainer, filepath)
-
-            # for model parallel we need to delete models for each model parallel rank
-            if self.last_model_path and self.last_model_path != filepath and app_state.data_parallel_rank == 0:
-                self._del_model(self.last_model_path)
-
-            self.last_model_path = filepath
-
-        else:
-            return super()._save_last_checkpoint(trainer, monitor_candidates)
-
-    def _save_none_monitor_checkpoint(self, trainer: 'pl.Trainer', monitor_candidates: Dict[str, _METRIC]) -> None:
-        """ Overrides PTL method to account for model parallel checkpoints.
-            Checks for data parallel rank 0 rather than global rank 0.
-        """
-        app_state = AppState()
-        if app_state.model_parallel_size is not None:
-            if self.monitor is not None or self.save_top_k == 0:
-                return
-
-            filepath = self._get_metric_interpolated_filepath_name(monitor_candidates, trainer)
-            self._save_model(trainer, filepath)
-
-            if (
-                self.save_top_k is None
-                and self.best_model_path
-                and self.best_model_path != filepath
-                and app_state.data_parallel_rank == 0
-            ):
-                self._del_model(self.best_model_path)
-
-            self.best_model_path = filepath
-        else:
-            return super()._save_none_monitor_checkpoint(trainer, monitor_candidates)
+        # each model parallel rank needs to remove its model
+        if is_global_rank_zero() or (app_state.model_parallel_size is not None and app_state.data_parallel_rank == 0):
+            try:
+                self._fs.rm(filepath)
+                logging.info(f"Removed checkpoint: {filepath}")
+            except:
+                logging.info(f"Tried to remove checkpoint: {filepath} but failed.")
 
 
 def configure_checkpointing(
@@ -882,16 +869,12 @@ def configure_checkpointing(
                 f"{trainer.check_val_every_n_epoch} epochs to ensure that checkpointing will not error out."
             )
 
-    if params.period is not None:
-        logging.warning(
-            "The use of `period` in the checkpoint callback is deprecrated, please use `every_n_val_epochs` instead. "
-            "Overwriting `every_n_val_epochs` with `period`."
-        )
-        params.every_n_val_epochs = params.period
-
-    # NOTE: checkpoint_callback should be called last
     checkpoint_callback = NeMoModelCheckpoint(n_resume=resume, **params)
-    checkpoint_callback.last_model_path = trainer.checkpoint_connector.resume_checkpoint_path or ""
+    checkpoint_callback.last_model_path = trainer.checkpoint_connector.resume_from_checkpoint_fit_path or ""
+    if params.model_parallel_size is not None and params.model_parallel_size > 1:
+        checkpoint_callback.last_model_path = NeMoModelCheckpoint._uninject_mp_rank(
+            checkpoint_callback.last_model_path
+        )
     trainer.callbacks.append(checkpoint_callback)
 
 
@@ -900,3 +883,32 @@ def check_slurm(trainer):
         return trainer.accelerator_connector.is_slurm_managing_tasks
     except AttributeError:
         return False
+
+
+class StatelessTimer(Timer):
+    """Extension of PTL timers to be per run."""
+
+    def __init__(self, duration: timedelta = None, interval: str = Interval.step, verbose: bool = True,) -> None:
+        super().__init__(duration, interval, verbose)
+
+    def on_save_checkpoint(self, trainer, pl_module, checkpoint) -> Dict[str, Any]:
+        return
+
+    def on_load_checkpoint(self, trainer, pl_module, callback_state) -> None:
+        return
+
+    def _check_time_remaining(self, trainer) -> None:
+        # Default timer only checks for train time exceeding max_time, this includes time for all stages.
+        train_duration = self.time_elapsed(RunningStage.TRAINING)
+        validation_duration = self.time_elapsed(RunningStage.VALIDATING)
+        test_duration = self.time_elapsed(RunningStage.TESTING)
+        total_duration = train_duration + validation_duration + test_duration
+        should_stop = total_duration >= self._duration
+        # should_stop = trainer.training_type_plugin.broadcast(should_stop)
+        should_stop = trainer.training_type_plugin.reduce_boolean_decision(should_stop)
+        trainer.should_stop = trainer.should_stop or should_stop
+        if should_stop and self._verbose:
+            rank_zero_info(f"Time limit reached. Signaling Trainer to stop.")
+            rank_zero_info(
+                f"Spent {timedelta(seconds=train_duration)} seconds on training, {timedelta(seconds=validation_duration)} seconds on validation and {timedelta(seconds=test_duration)} seconds on testing"
+            )
