@@ -13,9 +13,7 @@
 # limitations under the License.
 
 
-import csv
 import json
-import os
 import re
 from collections import defaultdict, namedtuple
 from typing import Dict, List, Optional, Set, Tuple
@@ -188,32 +186,6 @@ def training_data_to_sentences(data: List[Instance]) -> Tuple[List[str], List[st
     return un_normalized, normalized, categories
 
 
-def load_labels(abs_path):
-    """
-    loads relative path file as dictionary
-
-    Args:
-        abs_path: absolute path
-
-    Returns dictionary of mappings
-    """
-    label_tsv = open(abs_path)
-    labels = list(csv.reader(label_tsv, delimiter="\t"))
-    return labels
-
-
-def get_abs_path(rel_path):
-    """
-    Get absolute path
-
-    Args:
-        rel_path: relative path to this file
-        
-    Returns absolute path
-    """
-    return os.path.dirname(os.path.abspath(__file__)) + '/' + rel_path
-
-
 def post_process_punctuation(text: str) -> str:
     """
     Normalized quotes and spaces
