@@ -13,6 +13,7 @@
 # limitations under the License.
 
 import itertools
+import os
 from argparse import ArgumentParser
 from collections import OrderedDict
 from typing import List
@@ -28,6 +29,13 @@ try:
 except (ModuleNotFoundError, ImportError):
     PYNINI_AVAILABLE = False
 
+try:
+    from nemo.collections.common.tokenizers.moses_tokenizers import MosesProcessor
+
+    NLP_AVAILABLE = True
+except (ModuleNotFoundError, ImportError):
+    NLP_AVAILABLE = False
+
 
 class Normalizer:
     """
@@ -39,6 +47,7 @@ class Normalizer:
         lang: language specifying the TN rules, by default: English
         cache_dir: path to a dir with .far grammar file. Set to None to avoid using cache.
         overwrite_cache: set to True to overwrite .far files
+        whitelist: path to a file with whitelist replacements
     """
 
     def __init__(
@@ -48,6 +57,7 @@ class Normalizer:
         deterministic: bool = True,
         cache_dir: str = None,
         overwrite_cache: bool = False,
+        whitelist: str = None,
     ):
         assert input_case in ["lower_cased", "cased"]
 
@@ -64,12 +74,22 @@ class Normalizer:
             from nemo_text_processing.text_normalization.ru.verbalizers.verbalize_final import VerbalizeFinalFst
 
         self.tagger = ClassifyFst(
-            input_case=input_case, deterministic=deterministic, cache_dir=cache_dir, overwrite_cache=overwrite_cache
+            input_case=input_case,
+            deterministic=deterministic,
+            cache_dir=cache_dir,
+            overwrite_cache=overwrite_cache,
+            whitelist=whitelist,
         )
         self.verbalizer = VerbalizeFinalFst(deterministic=deterministic)
         self.parser = TokenParser()
         self.lang = lang
 
+        if NLP_AVAILABLE:
+            self.processor = MosesProcessor(lang_id=lang)
+        else:
+            self.processor = None
+            print("NeMo NLP is not available. Moses de-tokenization will be skipped.")
+
     def normalize_list(self, texts: List[str], verbose=False) -> List[str]:
         """
         NeMo text normalizer 
@@ -129,6 +149,9 @@ class Normalizer:
             output = self.select_verbalizer(verbalizer_lattice)
             if punct_post_process:
                 output = post_process_punctuation(output)
+                # do post-processing based on Moses detokenizer
+                if self.processor:
+                    output = self.processor.detokenize([output])
             return output
         raise ValueError()
 
@@ -256,6 +279,7 @@ def parse_args():
         "--punct_pre_process", help="set to True to enable punctuation pre processing", action="store_true"
     )
     parser.add_argument("--overwrite_cache", help="set to True to re-create .far grammar files", action="store_true")
+    parser.add_argument("--whitelist", help="path to a file with with whitelist", default=None, type=str)
     parser.add_argument(
         "--cache_dir",
         help="path to a dir with .far grammar file. Set to None to avoid using cache",
@@ -267,7 +291,10 @@ def parse_args():
 
 if __name__ == "__main__":
     args = parse_args()
-    normalizer = Normalizer(input_case=args.input_case, cache_dir=args.cache_dir, overwrite_cache=args.overwrite_cache)
+    whitelist = os.path.abspath(args.whitelist) if args.whitelist else None
+    normalizer = Normalizer(
+        input_case=args.input_case, cache_dir=args.cache_dir, overwrite_cache=args.overwrite_cache, whitelist=whitelist
+    )
     print(
         normalizer.normalize(
             args.input_string,
